#ifndef SAN_DRIVER_H__
#define SAN_DRIVER_H__

extern void PMIC_SM_27A(void);
extern void PMIC_SM_27B(void);
extern void PMIC_SM_27C(void);
extern void PMIC_SM_27D(void);
extern void check_SoC_Activation(void);
extern void PMIC_SM_12_wdt(void);

extern void SM_6_3_1(void);
extern void SM_6_3_2(void);
extern void SM_5_3_6(void);
extern void SM_5_1(void);
extern void SM_6_23(void);
extern void SM_4_21(void);
extern void SM_5_1_6(void);
extern void SM_5_2_6(void);
extern void SM_6_2(void);
extern void SM_6_11(void);
extern void SM_6_14_6(void);
extern void SM_6_22(void);
extern void SM_6_24_2(void);
extern void SM_6_24_3(void);
extern void SM_6_29(void);
extern void SM_6_30(void);

#endif  /* SAN_DRIVER_H__ */

